/*
 * tccext.h - Parse information received from tcc's external interface
 *
 * Written 2001 by Werner Almesberger
 * Copyright 2001 Network Robots.
 * Distributed under the LGPL.
 */


#ifndef TCCEXT_H
#define TCCEXT_H


#include <stdint.h>
#include <stdio.h>


#define __TE_SIZEOF(user_size,default_size) \
  ({ if ((user_size) && (user_size) < (default_size)) abort(); \
    (user_size) ? (user_size) : (default_size); })

#define TE_SIZEOF(ctx,field,dflt) __TE_SIZEOF(ctx->sizes.field,sizeof(dflt))


struct _tccext_offset;

typedef struct _tccext_field {
    struct _tccext_offset *offset_group;/* base offset group; NULL for 0/raw */
    int offset;				/* bit offset within offset group */
    int length;				/* field length in bits */
} TCCEXT_FIELD;

typedef struct _tccext_offset {
    struct _tccext_offset *base;	/* base offset group; NULL for 0/raw */
    TCCEXT_FIELD field;			/* field containing increment */
    int shift_left;			/* increment scaling (to yield bits) */
    int group_number;			/* group number (semi-private) */
    struct _tccext_offset *next;	/* next offset group; NULL if last
					   (semi-private) */
} TCCEXT_OFFSET;

typedef struct _tccext_match {
    TCCEXT_FIELD field;			/* field to match with */
    uint8_t *data;			/* data, right-aligned to byte
					   boundary */
    struct _tccext_match *next;		/* next match in rule; NULL if last */
    struct _tccext_context *context;	/* back pointer; PRIVATE */
} TCCEXT_MATCH;

typedef struct _tccext_rule {
    TCCEXT_MATCH *matches;		/* ordered list of matches */
    int action;				/* action code */
    struct _tccext_rule *next;		/* next rule; NULL if last */
} TCCEXT_RULE;

typedef struct _tccext_queue {
    /* TBD */
    struct _tccext_queue *next;		/* next queue; NULL if last */
} TCCEXT_QUEUE;

typedef struct {
    int offset;				/* user offset size; 0 for default */
    int match;				/* user match size; 0 for default */
    int rule;				/* user rule size; 0 for default */
    int queue;				/* user queue size; 0 for default */
    int context;			/* user context size; 0 dflt */
} TCCEXT_SIZES;

typedef struct _tccext_context {
    TCCEXT_OFFSET *offset_groups;	/* offset groups (semi-private) */
    TCCEXT_RULE *rules;			/* ordered list of rules; NULL if
					   none */
    TCCEXT_QUEUE *queues;		/* ordered list of queues; NULL if
					   none */
    TCCEXT_SIZES sizes;
} TCCEXT_CONTEXT;


TCCEXT_CONTEXT *tccext_parse(FILE *file,TCCEXT_SIZES *sizes);
    /* pass NULL in sizes to use defaults */

void tccext_destroy(TCCEXT_CONTEXT *ctx);

#endif
